// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package garden

import (
	"context"
	"encoding/json"
	"fmt"
	"gorm.io/gorm"
	"property-system/errors"
	dbmodel "property-system/model"
	pb_v1 "property-system/pb/v1"
	"time"

	"git.getensh.com/common/gopkgs/database"
	"git.getensh.com/common/gopkgs/logger"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
)

func checkGardenChangeCompanyParam(req *pb_v1.GardenChangeCompanyRequest) error {
	switch {
	case req.Cid < 1:
		return status.Error(10003, "物业公司不能为空")
	case req.GardenId < 1:
		return status.Error(10003, "请选择要更改的小区")

	}
	return nil
}



func unbindPropertySystemUser(db *gorm.DB, gardenId int64, cid int64) error {
	user := dbmodel.TUser{}
	where := map[string]interface{}{
		"garden_id":gardenId,
		"cid":cid,
	}
	values := map[string]interface{}{
		"garden_id":0,
		"group_id":0,
		"updated_at":time.Now(),
	}
	err := user.Update(db, where, values)
	if err != nil {
		return errors.DataBaseError
	}
	return nil
}

func rebindPropertySystemGroup(db *gorm.DB, gardenId int64, oldCid int64, newCid int64) error {
	group := dbmodel.TGroup{}
	where := map[string]interface{}{
		"garden_id":gardenId,
		"cid":oldCid,
	}
	values := map[string]interface{}{
		"cid":newCid,
		"updated_at":time.Now(),
	}
	err := group.Update(db, where, values)
	if err != nil {
		return errors.DataBaseError
	}
	return nil
}

//
func GardenChangeCompany(ctx context.Context, req *pb_v1.GardenChangeCompanyRequest) (reply *pb_v1.GardenChangeCompanyReply, err error) {

	reply = &pb_v1.GardenChangeCompanyReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()

	err = checkGardenChangeCompanyParam(req)
	if err != nil {
		return nil, err
	}

	p := &dbmodel.TGardenApproved{}
	where := map[string]interface{}{
		"id":req.GardenId,
	}
	err = p.Find(database.DB(), where)
	if err != nil && err != gorm.ErrRecordNotFound {
		return nil, errors.DataBaseError
	}
	if p.ID == 0 {
		return nil, errors.GardenNotExist
	}
	if p.Cid == req.Cid {
		return nil, status.Error(10003, "物业公司未改变")
	}
	reply.Origin = &pb_v1.GardenChangeCompanyRequest{
		GardenId:p.ID,
		Cid:p.Cid,
	}

	oldCid := p.Cid
	values := map[string]interface{}{
		"cid":req.Cid,
	}
	db := database.DB().Begin()
	err = p.Update(db, where, values)
	if err != nil {
		db.Rollback()
		return nil, errors.DataBaseError
	}
	// 解除物业用户与该小区的绑定关系
	err = unbindPropertySystemUser(db, req.GardenId, oldCid)
	if err != nil {
		db.Rollback()
		return nil, err
	}
	// 重新绑定小区权限组与公司
	err = rebindPropertySystemGroup(db, req.GardenId, oldCid, req.Cid)
	if err != nil {
		db.Rollback()
		return nil, err
	}

	db.Commit()

	return reply, nil
}

