// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package garden

import (
	"context"
	"encoding/json"
	"fmt"
	"gorm.io/gorm"
	"property-system/errors"
	dbmodel "property-system/model"
	"property-system/pb"
	pb_v1 "property-system/pb/v1"
	"time"

	"git.getensh.com/common/gopkgs/database"
	"git.getensh.com/common/gopkgs/logger"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
)
type KeyInfoChangeContent struct {
	Key        string `json:"key"`
	KeyChinese string `json:"key_chinese"`
	OldValue   string `json:"old_value"`
	NewValue   string `json:"new_value"`
}

func checkGardenKeyInfoChangeParam(req *pb_v1.GardenKeyInfoChangeRequest) error {
	switch {
	case req.Province == "" || req.ProvinceCode == "":
		return status.Error(10003, "省份不能为空")
	case req.City == "" || req.CityCode == "":
		return status.Error(10003, "城市不能为空")
	case req.Area == "" || req.AreaCode == "":
		return status.Error(10003, "区域不能为空")
	case req.Street == "" || req.StreetCode == "":
		return status.Error(10003, "街道不能为空")
	case req.Committee == "" || req.CommitteeCode == "":
		return status.Error(10003, "社区不能为空")
	case req.Cid < 1:
		return status.Error(10003, "物业公司不能为空")
	case req.Id < 1:
		return status.Error(10003, "请选择要更改的小区")
	case req.GardenName == "":
		return status.Error(10003, "小区不能为空")

	}
	return nil
}



//
func GardenKeyInfoChange(ctx context.Context, req *pb_v1.GardenKeyInfoChangeRequest) (reply *pb_v1.GardenKeyInfoChangeReply, err error) {

	reply = &pb_v1.GardenKeyInfoChangeReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()

	err = checkGardenKeyInfoChangeParam(req)
	if err != nil {
		return nil, err
	}

	p := &dbmodel.TGardenApproved{}
	where := map[string]interface{}{
		"id":req.Id,
	}
	err = p.Find(database.DB(), where)
	if err != nil && err != gorm.ErrRecordNotFound {
		return nil, errors.DataBaseError
	}
	if p.ID == 0 {
		return nil, errors.GardenNotExist
	}
	if p.Cid != req.Cid {
		return nil, status.Error(10003, "无法操作该小区")
	}
	//addrChange := false
	if req.CommitteeCode != p.CommitteeCode {

		mreq := pb_v1.GardenHouseIsInRequest{GardenId:req.Id}
		mreply, err := pb.Garden.GardenHouseIsIn(ctx, &mreq)
		if err != nil {
			return nil, err
		}
		if mreply.In  {
			return nil, status.Error(10003, "已入住房屋, 不能修改小区地址信息")
		}

		//addrChange = true
	}

	array := []KeyInfoChangeContent{}
	if req.GardenName != p.GardenName {
		item := KeyInfoChangeContent{
			Key:"garden_name",
			KeyChinese:"小区名",
			OldValue:p.GardenName,
			NewValue:req.GardenName,
		}
		array = append(array, item)
	}
	if req.ProvinceCode != p.ProvinceCode {
		item := KeyInfoChangeContent{
			Key:"province_code",
			KeyChinese:"省份代码",
			OldValue:p.ProvinceCode,
			NewValue:req.ProvinceCode,
		}
		array = append(array, item)
		item = KeyInfoChangeContent{
			Key:"province",
			KeyChinese:"省份",
			OldValue:p.Province,
			NewValue:req.Province,
		}
		array = append(array, item)
	}

	if req.CityCode != p.CityCode {
		item := KeyInfoChangeContent{
			Key:"cit_code",
			KeyChinese:"城市代码",
			OldValue:p.CityCode,
			NewValue:req.CityCode,
		}
		array = append(array, item)
		item = KeyInfoChangeContent{
			Key:"city",
			KeyChinese:"城市",
			OldValue:p.City,
			NewValue:req.City,
		}
		array = append(array, item)
	}

	if req.AreaCode != p.AreaCode {
		item := KeyInfoChangeContent{
			Key:"area_code",
			KeyChinese:"区域代码",
			OldValue:p.AreaCode,
			NewValue:req.AreaCode,
		}
		array = append(array, item)
		item = KeyInfoChangeContent{
			Key:"area",
			KeyChinese:"区域",
			OldValue:p.Area,
			NewValue:req.Area,
		}
		array = append(array, item)
	}

	if req.StreetCode != p.StreetCode {
		item := KeyInfoChangeContent{
			Key:"street_code",
			KeyChinese:"街道代码",
			OldValue:p.StreetCode,
			NewValue:req.StreetCode,
		}
		array = append(array, item)
		item = KeyInfoChangeContent{
			Key:"street",
			KeyChinese:"街道",
			OldValue:p.Street,
			NewValue:req.Street,
		}
		array = append(array, item)
	}

	if req.CommitteeCode != p.CommitteeCode {
		item := KeyInfoChangeContent{
			Key:"committe_code",
			KeyChinese:"省份代码",
			OldValue:p.CommitteeCode,
			NewValue:req.CommitteeCode,
		}
		array = append(array, item)
		item = KeyInfoChangeContent{
			Key:"committe",
			KeyChinese:"省份",
			OldValue:p.Committee,
			NewValue:req.Committee,
		}
		array = append(array, item)
	}

	if len(array) == 0 {
		return reply, nil
	}
	content, _ := json.Marshal(array)
	now := time.Now()
	gChange := dbmodel.TGardenChange{
		Content:string(content),
		Status:1,
		CreatedAt:now,
		UpdatedAt:now,
		Cid:req.Cid,
		GardenId:req.Id,
	}
	err = gChange.Insert(database.DB())
	if err != nil {
		return nil, errors.DataBaseError
	}
	return reply, nil

}

