// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package garden

import (
	"context"
	"encoding/json"
	"fmt"
	"property-system/errors"
	dbmodel "property-system/model"

	pb_v1 "property-system/pb/v1"

	"git.getensh.com/common/gopkgs/database"
	"git.getensh.com/common/gopkgs/logger"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
)

func checkGardenKeyInfoChangeListParam(req *pb_v1.GardenKeyInfoChangeListRequest) error {
	if req.Page == 0 {
		req.Page = 1
	}
	if req.PageSize == 0 {
		req.PageSize = 1
	}
	return nil
}

func GetGardenInfos(ids []int64) (map[int64]dbmodel.TGardenApproved, error) {
	ret := map[int64]dbmodel.TGardenApproved{}
	if len(ids) == 0 {
		return ret, nil
	}
	p := dbmodel.TGardenApproved{}
	where := map[string]interface{}{
		"id in": ids,
	}
	list, err := p.List(database.DB(), where, nil, -1, -1)
	if err != nil {
		return nil, errors.DataBaseError
	}
	for _, v := range list {
		ret[v.ID] = v
	}
	return ret, nil
}

//
func GardenKeyInfoChangeList(ctx context.Context, req *pb_v1.GardenKeyInfoChangeListRequest) (reply *pb_v1.GardenKeyInfoChangeListReply, err error) {

	reply = &pb_v1.GardenKeyInfoChangeListReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()

	err = checkGardenKeyInfoChangeListParam(req)
	if err != nil {
		return nil, err
	}

	// 获取修改信息
	p := &dbmodel.TGardenChange{}
	where := map[string]interface{}{}
	if req.Cid > 0 {
		where["cid"] = req.Cid
	}
	if req.GardenId > 0 {
		where["garden_id"] = req.GardenId
	}
	if req.Status > 0 {
		where["status"] = req.Status
	}
	reply.Page = req.Page
	reply.Total, err = p.Count(database.DB(), where, nil)
	if err != nil {
		return nil, errors.DataBaseError
	}
	if reply.Total == 0 {
		return reply, nil
	}
	list, err := p.List(database.DB(), where, nil, int(req.Page), int(req.PageSize))
	if err != nil {
		return nil, errors.DataBaseError
	}
	gardenIds := []int64{}
	gardenIdM := map[int64]bool{}
	for _, v := range list {
		if gardenIdM[v.GardenId] {
			continue
		}
		gardenIdM[v.GardenId] = true
		gardenIds = append(gardenIds, v.GardenId)
	}
	gardenInfosM, err := GetGardenInfos(gardenIds)
	if err != nil {
		return nil, err
	}

	reply.List = make([]*pb_v1.GardenKeyInfoData, len(list))
	for i, v := range list {
		gardenInfo := gardenInfosM[v.GardenId]
		item := &pb_v1.GardenKeyInfoData{
			GardenId:   v.GardenId,
			Cid:        v.Cid,
			Id:         v.ID,
			GardenName: gardenInfo.GardenName,
			GardenDetail: fmt.Sprintf("%s%s%s%s%s",
				gardenInfo.Province, gardenInfo.City, gardenInfo.Area, gardenInfo.Street, gardenInfo.Committee),
			Status:   v.Status,
			Feedback: v.Feedback,
		}

		array := []KeyInfoChangeContent{}
		json.Unmarshal([]byte(v.Content), &array)

		item.List = make([]*pb_v1.GardenKeyInfoItem, len(array))
		for j, _ := range array {
			item.List[j] = &pb_v1.GardenKeyInfoItem{
				FieldName:     array[j].KeyChinese,
				FieldValueSrc: array[j].OldValue,
				FieldValueDst: array[j].NewValue,
			}
		}
		reply.List[i] = item
	}

	return reply, nil
}
