// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package garden

import (
	"context"
	"encoding/json"
	"fmt"
	"gorm.io/gorm"
	"property-system/errors"
	"property-system/impl/v1/oss_utils"
	dbmodel "property-system/model"
	"property-system/pb"
	pb_v1 "property-system/pb/v1"
	"strings"
	"time"

	"git.getensh.com/common/gopkgs/database"
	"git.getensh.com/common/gopkgs/logger"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
)

func checkGardenUpdateParam(req *pb_v1.GardenUpdateRequest) error {
	switch {
	//case req.Province == "" || req.ProvinceCode == "":
	//	return status.Error(10003, "省份不能为空")
	//case req.City == "" || req.CityCode == "":
	//	return status.Error(10003, "城市不能为空")
	//case req.Area == "" || req.AreaCode == "":
	//	return status.Error(10003, "区域不能为空")
	//case req.Street == "" || req.StreetCode == "":
	//	return status.Error(10003, "街道不能为空")
	//case req.Committee == "" || req.CommitteeCode == "":
	//	return status.Error(10003, "社区不能为空")
	case req.PropertyPerson == "":
		return status.Error(10003, "物业联系人不能为空")
	case req.PropertyPhone == "":
		return status.Error(10003, "物业联系人电话不能为空")
	//case req.GardenName == "":
	//	return status.Error(10003, "小区名称不能为空")
	case req.GardenAddr == "":
		return status.Error(10003, "小区地址不能为空")
	case req.Cid < 1:
		return status.Error(10003, "物业公司不能为空")
	case req.Id < 1:
		return status.Error(10003, "请选择要更改的小区")
	case req.Lnt == 0 || req.Lat == 0:
		return status.Error(10003, "经纬度不能为空")
	case (req.BuildingStart > 0 && req.BuildingEnd < 1) ||
		(req.BuildingStart < 1 && req.BuildingEnd > 0):
		return status.Error(10003, "建成年份只能同时为空或不为空")
	case (req.PropertyFeeStart > 0 && req.PropertyFeeEnd < 1) ||
		(req.PropertyFeeStart < 1 && req.PropertyFeeEnd > 0):
		return status.Error(10003, "物业费只能同时为空或不为空")
	case (req.GasFeeStart > 0 && req.GasFeeEnd < 1) ||
		(req.GasFeeStart < 1 && req.GasFeeEnd > 0):
		return status.Error(10003, "燃气费只能同时为空或不为空")

	}
	return nil
}

//
func GardenUpdate(ctx context.Context, req *pb_v1.GardenUpdateRequest) (reply *pb_v1.GardenUpdateReply, err error) {

	reply = &pb_v1.GardenUpdateReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()

	err = checkGardenUpdateParam(req)
	if err != nil {
		return nil, err
	}

	p := &dbmodel.TGardenApproved{}
	where := map[string]interface{}{
		"id": req.Id,
	}
	err = p.Find(database.DB(), where)
	if err != nil && err != gorm.ErrRecordNotFound {
		return nil, errors.DataBaseError
	}
	if p.ID == 0 {
		return nil, errors.GardenNotExist
	}
	if p.Cid != req.Cid {
		return nil, status.Error(10003, "无法操作该小区")
	}

	reply.Origin = &pb_v1.GardenUpdateRequest{
		Id:             req.Id,
		GardenName:     p.GardenName,
		GardenDesc:     p.GardenDesc,
		StreetCode:     p.StreetCode,
		CommitteeCode:  p.CommitteeCode,
		ProvinceCode:   p.ProvinceCode,
		CityCode:       p.CityCode,
		AreaCode:       p.AreaCode,
		Cid:            p.Cid,
		PropertyPerson: p.PropertyPerson,
		PropertyPhone:  p.PropertyPhone,
		GardenPic:      p.GardenPic,
		GardenAddr:     p.GardenAddr,
		Lnt:            p.Lnt,
		Lat:            p.Lat,
	}

	now := time.Now()

	garden := dbmodel.TGardenApproved{}
	values := map[string]interface{}{
		//"province":       req.Province,
		//"province_code":   req.ProvinceCode,
		//"city":           req.City,
		//"city_code":       req.CityCode,
		//"area":           req.Area,
		//"area_code":       req.AreaCode,
		//"street":         req.Street,
		//"street_code":     req.StreetCode,
		//"committee":      req.Committee,
		//"committee_code":  req.CommitteeCode,
		"property_person": req.PropertyPerson,
		"property_phone":  req.PropertyPhone,
		//"garden_name":     req.GardenName,
		"garden_addr":        req.GardenAddr,
		"cid":                req.Cid,
		"garden_pic":         req.GardenPic,
		"garden_desc":        req.GardenDesc,
		"in_use":             true,
		"updated_at":         now,
		"lat":                req.Lat,
		"lnt":                req.Lnt,
		"garden_pics":        strings.Join(req.GardenPics, ";"),
		"building_start":     req.BuildingStart,
		"building_end":       req.BuildingEnd,
		"property_fee_start": req.PropertyFeeStart,
		"property_fee_end":   req.PropertyFeeEnd,
		"gas_fee_start":      req.GasFeeStart,
		"gas_fee_end":        req.GasFeeEnd,
		"building_area":      req.BuildingArea,
		"building_company":   req.BuildingCompany,
		"covered_area":       req.CoveredArea,
		"green_percent":      req.GreenPercent,
		"area_percent":       req.AreaPercent,
		"space_info":         req.SpaceInfo,
		"space_total":        req.SpaceTotal,
		"house_total":        req.HouseTotal,
		"building_type":      req.BuildingType,
		"water_type":         int32ArrayToString(req.WaterType),
		"electric_type":      int32ArrayToString(req.ElectricType),
		"avg_price":          req.AvgPrice,
	}
	where = map[string]interface{}{
		"id": req.Id,
	}

	db := database.DB().Begin()
	defer func() {
		if err != nil {
			db.Rollback()
		}
	}()
	err = garden.Update(db, where, values)
	if err != nil {
		if strings.Contains(strings.ToLower(err.Error()), "duplicate") {

			return nil, errors.GardenExist
		}

		return nil, errors.DataBaseError
	}

	if reply.Origin.Lat != req.Lat || reply.Origin.Lnt != req.Lnt {
		mreq := pb_v1.GardenHouseRentChangeFieldRequest{
			GardenId:   req.Id,
			GardenName: req.GardenName,
			Lat:        req.Lat,
			Lnt:        req.Lnt,
		}

		_, err = pb.Garden.GardenHouseRentChangeField(ctx, &mreq)
		if err != nil {
			return nil, err
		}
	}
	inlist := []string{req.GardenPic}
	outlist := []string{p.GardenPic}
	inlist = append(inlist, req.GardenPics...)
	outlist = append(outlist, strings.Split(p.GardenPics, ";")...)
	if err := oss_utils.OssObjAdd(inlist, outlist); err != nil {
		db.Rollback()
		return nil, err
	}

	db.Commit()

	return reply, nil
}
