// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package msg_package

import (
	"context"
	"encoding/json"
	"fmt"
	"git.getensh.com/common/gopkgs/database"
	"git.getensh.com/common/gopkgs/logger"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
	"property-system/errors"
	dbmodel "property-system/model"
	pb_v1 "property-system/pb/v1"
)

func checkMsgPackageOrderListParam(req *pb_v1.MsgPackageOrderListRequest) error {
	if req.PageSize == 0 {
		req.PageSize = 10
	}
	if req.Page == 0 {
		req.Page = 1
	}
	if req.Management {
		return nil
	}
	if req.GardenId < 0 {
		return status.Error(10003, "小区不能为空")
	}
	if req.Available {
		req.Status = 2
		req.Page = -1
		req.PageSize = -1
		return nil
	}

	return nil
}

//
func MsgPackageOrderList(ctx context.Context, req *pb_v1.MsgPackageOrderListRequest) (reply *pb_v1.MsgPackageOrderListReply, err error) {

	reply = &pb_v1.MsgPackageOrderListReply{}
	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()

	err = checkMsgPackageOrderListParam(req)
	if err != nil {
		return nil, err
	}

	p := dbmodel.TMsgPackageOrder{}
	where := map[string]interface{}{}

	if req.GardenId > 0 {
		where["garden_id"] = req.GardenId
	}
	if req.Status > 0 {
		where["status"] = req.Status
	}
	order := "created_at desc"
	if req.Available {
		order = "created_at"
		where["msg_available_count >"] = 0
	}

	reply.Total, err = p.Count(database.DB(), where, nil)
	if err != nil {
		return nil, errors.DataBaseError
	}
	reply.Page = req.Page
	if reply.Total == 0 {
		return reply, nil
	}

	list, err := p.List(database.DB(), where, nil, int(req.Page), int(req.PageSize), order)
	if err != nil {
		return nil, errors.DataBaseError
	}
	gardenIds := []int64{}
	garden := dbmodel.TGardenApproved{}
	for _, v := range list {
		if v.GardenId > 0 {
			gardenIds = append(gardenIds, v.GardenId)
		}

	}
	where = map[string]interface{}{
		"id in": gardenIds,
	}
	gardenM := map[int64]dbmodel.TGardenApproved{}
	if len(gardenIds) > 0 {
		glist, err := garden.List(database.DB(), where, nil, -1, -1)
		if err != nil {
			return nil, errors.DataBaseError
		}
		for _, v := range glist {
			gardenM[v.ID] = v
		}
	}

	reply.List = make([]*pb_v1.MsgPackageOrderItem, len(list))

	for i, v := range list {
		item := &pb_v1.MsgPackageOrderItem{
			Name:              v.Name,
			Amount:            v.Amount,
			MsgTotalCount:     v.MsgTotalCount,
			MsgCount:          v.MsgCount,
			MsgAvailableCount: v.MsgAvailableCount,
			PackageId:         v.PackageId,
			OrderId:           v.ID,
			Status:            v.Status,
			PackageCount:      v.PackageCount,
			Price:             v.Price,
			GardenId:          v.GardenId,
			Comment:           v.Comment,
			Feedback:          v.Feedback,
			GardenName:        gardenM[v.GardenId].GardenName,
			CreatedAt:         v.CreatedAt.Unix(),
		}

		reply.List[i] = item
	}

	return reply, nil
}
