// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package permission

import (
	"context"
	"encoding/json"
	"fmt"
	"git.getensh.com/common/gopkgs/cache"
	"git.getensh.com/common/gopkgs/database"
	"git.getensh.com/common/gopkgs/logger"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
	"gorm.io/gorm"
	"property-system/errors"
	dbmodel "property-system/model"
	"property-system/pb"
	pb_v1 "property-system/pb/v1"
	"strings"
)

func checkGardenPermissionListParam(req *pb_v1.GardenPermissionListRequest) error {
	if req.GardenId < 1 {
		return errors.ParamsError
	}
	return nil
}

const gardenPermissionTreePrefix = "garden_permissions"
const gardenPermissionCodesPrefix = "garden_permission_codes"

func gardenPermissionTreeKey(gardenId int64) string {
	return fmt.Sprintf("%s_%d", gardenPermissionTreePrefix, gardenId)
}

func GardenPermissionTreeFromRedis(gardenId int64) []*pb_v1.SystemPermissionItem {
	key := gardenPermissionTreeKey(gardenId)
	str, _ := cache.Redis().Get(key)
	if str == "" {
		return nil
	}
	ret := []*pb_v1.SystemPermissionItem{}
	json.Unmarshal([]byte(str), &ret)
	return ret
}

func GardenPermissionTreeToRedis(gardenId int64, permissions []*pb_v1.SystemPermissionItem) {
	key := gardenPermissionTreeKey(gardenId)
	if len(permissions) == 0 {
		return
	}
	bytes, _ := json.Marshal(permissions)
	cache.Redis().SetEx(key, 600, string(bytes))
}

func GardenPermissionTreeClearRedis(gardenId int64) {
	key := gardenPermissionTreeKey(gardenId)
	GlobalPermissionTimeCache(gardenId)
	GardenPermissionCodesClearRedis(gardenId)
	cache.Redis().Del(key)
}

func makeGardenPermissionTree(src []*pb_v1.SystemPermissionItem, gardenCodes map[string]bool) []*pb_v1.SystemPermissionItem {
	ret := make([]*pb_v1.SystemPermissionItem, 0)
	for _, v := range src {
		if !gardenCodes[v.Code] {
			continue
		}
		item := &pb_v1.SystemPermissionItem{
			Pcode:       v.Pcode,
			Code:        v.Code,
			Name:        v.Name,
			Router:      v.Router,
			Path:        v.Path,
			AppletePath: v.AppletePath,
			Level:       v.Level,
			CreatedAt:   v.CreatedAt,
		}
		if len(v.Childs) > 0 {
			item.Childs = makeGardenPermissionTree(v.Childs, gardenCodes)
		}
		ret = append(ret, item)
	}
	return ret
}

func GardenPermissionTreeFromDb(gardenId int64) ([]*pb_v1.SystemPermissionItem, error) {

	mreq := &pb_v1.SystemPermissionListRequest{}
	mreply, err := pb.Common.SystemPermissionList(context.Background(), mreq)
	if err != nil {
		return nil, err
	}

	gardenCodesM, err := GardenPermissionCodes(gardenId)
	ret := makeGardenPermissionTree(mreply.List, gardenCodesM)
	GardenPermissionTreeToRedis(gardenId, ret)
	return ret, nil
}

func GardenPermissionTree(gardenId int64) ([]*pb_v1.SystemPermissionItem, error) {
	ret := GardenPermissionTreeFromRedis(gardenId)
	if len(ret) > 0 {
		return ret, nil
	}
	return GardenPermissionTreeFromDb(gardenId)
}

func gardenPermissionCodesKey(gardenId int64) string {
	return fmt.Sprintf("%s_%d", gardenPermissionCodesPrefix, gardenId)
}

func GardenPermissionCodesFromRedis(gardenId int64) map[string]bool {
	ret := map[string]bool{}
	key := gardenPermissionCodesKey(gardenId)
	str, _ := cache.Redis().Get(key)
	if str == "" {
		return nil
	}
	json.Unmarshal([]byte(str), &ret)
	return ret
}

func GardenPermissionCodesToRedis(gardenId int64, codes map[string]bool) {
	key := gardenPermissionCodesKey(gardenId)
	if len(codes) == 0 {
		return
	}
	bytes, _ := json.Marshal(codes)
	cache.Redis().SetEx(key, 600, string(bytes))
}

func GardenPermissionCodesClearRedis(gardenId int64) {
	key := gardenPermissionCodesKey(gardenId)
	cache.Redis().Del(key)
}

func GardenPermissionCodesFromDb(gardenId int64) (map[string]bool, error) {
	p := &dbmodel.TGardenApproved{}
	where := map[string]interface{}{
		"id": gardenId,
	}
	err := p.Find(database.DB(), where)
	if err != nil && err != gorm.ErrRecordNotFound {
		return nil, errors.DataBaseError
	}
	if p.ID == 0 {
		return nil, errors.ErrRecordNotFound
	}

	gardenCodes := strings.Split(p.PermissionCodes, ";")
	gardenCodesM := map[string]bool{}
	for _, v := range gardenCodes {
		if v == "" {
			continue
		}
		gardenCodesM[v] = true
	}

	GardenPermissionCodesToRedis(gardenId, gardenCodesM)
	return gardenCodesM, nil
}

func GardenPermissionCodes(gardenId int64) (map[string]bool, error) {
	ret := GardenPermissionCodesFromRedis(gardenId)
	if len(ret) > 0 {
		return ret, nil
	}
	return GardenPermissionCodesFromDb(gardenId)
}

func convertGardenPermission(src []*pb_v1.SystemPermissionItem) []*pb_v1.GardenPermissionItem {
	ret := make([]*pb_v1.GardenPermissionItem, len(src))
	for i, v := range src {
		ret[i] = &pb_v1.GardenPermissionItem{
			Pcode:       v.Pcode,
			Code:        v.Code,
			Name:        v.Name,
			Router:      v.Router,
			Path:        v.Path,
			AppletePath: v.AppletePath,
			Level:       v.Level,
			CreatedAt:   v.CreatedAt,
		}
		if len(v.Childs) > 0 {
			ret[i].Childs = convertGardenPermission(v.Childs)
		}
	}
	return ret
}

//
func GardenPermissionList(ctx context.Context, req *pb_v1.GardenPermissionListRequest) (reply *pb_v1.GardenPermissionListReply, err error) {

	reply = &pb_v1.GardenPermissionListReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()

	err = checkGardenPermissionListParam(req)
	if err != nil {
		return nil, err
	}

	list, err := GardenPermissionTree(req.GardenId)
	if err != nil {
		return nil, err
	}

	reply.List = convertGardenPermission(list)

	return reply, nil
}
