// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package permission

import (
	"context"
	"encoding/json"
	"fmt"
	"git.getensh.com/common/gopkgs/database"
	"git.getensh.com/common/gopkgs/logger"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
	"property-system/errors"
	dbmodel "property-system/model"
	"property-system/pb"
	pb_v1 "property-system/pb/v1"
	"strings"
	"time"
)

type Permission struct {
	Code        string
	Router      string
	Path        string
	Level       int64
	Enable      int32
	AppletePath string
}

func checkGroupAddParam(req *pb_v1.GroupAddRequest) error {
	switch {
	case len(req.PermissionCodes) == 0 && !req.IsSuper:
		return status.Error(10003, "普通权限组权限不能为空")
	case req.GardenId < 1:
		return status.Error(10003, "小区不能为空")
	case req.Cid < 1:
		return status.Error(10003, "物业公司不能为空")
	case req.GroupName == "":
		return status.Error(10003, "权限组名称不能为空")

	}
	return nil
}

func getPermissions(codes []string) (map[string]Permission, error) {
	ret := map[string]Permission{}
	m := map[string]bool{}
	mreq := &pb_v1.SystemPermissionsNotTreeRequest{}
	mreply, err := pb.Common.SystemPermissionsNotTree(context.Background(), mreq)
	if err != nil {
		return nil, err
	}
	for _, v := range codes {
		m[v] = true
	}
	for _, v := range mreply.List {
		if v.Enable == 2 {
			continue
		}
		if m[v.Code] {
			ret[v.Code] = Permission{
				Code:        v.Code,
				Router:      v.Router,
				Path:        v.Path,
				AppletePath: v.AppletePath,
				Level:       v.Level,
			}
		}
	}
	return ret, nil

}

//
func GroupAdd(ctx context.Context, req *pb_v1.GroupAddRequest) (reply *pb_v1.GroupAddReply, err error) {

	reply = &pb_v1.GroupAddReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()

	err = checkGroupAddParam(req)
	if err != nil {
		return nil, err
	}

	if req.IsSuper {
		p := dbmodel.TGroup{}
		where := map[string]interface{}{
			"cid":       req.Cid,
			"garden_id": req.GardenId,
		}
		count, err := p.Count(database.DB(), where, nil)
		if err != nil {
			return nil, errors.DataBaseError
		}
		if count > 0 {
			return reply, nil
		}
	}

	now := time.Now()
	isSuper := 1
	if !req.IsSuper {
		isSuper = 2
	}

	gardenCodesM, err := GardenPermissionCodes(req.GardenId)
	if err != nil {
		return nil, err
	}
	array := []string{}
	for _, v := range req.PermissionCodes {
		if gardenCodesM[v] {
			array = append(array, v)
		}
	}
	req.PermissionCodes = array

	codes := strings.Join(req.PermissionCodes, ",")

	if len(req.PermissionCodes) == 0 {
		codes = ""
	} else {
		m, err := getPermissions(req.PermissionCodes)
		if err != nil {
			return nil, err
		}
		bytes, _ := json.Marshal(m)
		codes = string(bytes)
	}

	group := &dbmodel.TGroup{
		GardenId:        req.GardenId,
		Cid:             req.Cid,
		PermissionCodes: string(codes),
		IsSuper:         int64(isSuper),
		GroupName:       req.GroupName,
		UpdatedAt:       now,
		CreatedAt:       now,
		GroupDesc:       req.GroupDesc,
	}
	err = group.Insert(database.DB())

	if err != nil {
		return nil, errors.DataBaseError
	}
	reply.Id = group.ID

	return reply, nil
}
