// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package permission

import (
	"context"
	"encoding/json"
	"fmt"
	"git.getensh.com/common/gopkgs/database"
	"git.getensh.com/common/gopkgs/logger"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
	"gorm.io/gorm"
	"property-system/errors"
	dbmodel "property-system/model"
	pb_v1 "property-system/pb/v1"
)

func makeSuperGroupPermission(src []*pb_v1.SystemPermissionItem) []*pb_v1.SystemGroupPermissionItem {
	ret := make([]*pb_v1.SystemGroupPermissionItem, len(src))
	for i, v := range src {
		ret[i] = &pb_v1.SystemGroupPermissionItem{
			Pcode:       v.Pcode,
			Code:        v.Code,
			Name:        v.Name,
			Router:      v.Router,
			Path:        v.Path,
			AppletePath: v.AppletePath,
			Selected:    true,
		}
		if len(v.Childs) > 0 {
			ret[i].Childs = makeSuperGroupPermission(v.Childs)
		}
	}
	return ret
}

func makeNormalGroupPermission(src []*pb_v1.SystemPermissionItem, codes map[string]Permission, usedCodes map[string]Permission) []*pb_v1.SystemGroupPermissionItem {
	ret := make([]*pb_v1.SystemGroupPermissionItem, len(src))
	for i, v := range src {
		ret[i] = &pb_v1.SystemGroupPermissionItem{
			Pcode:       v.Pcode,
			Code:        v.Code,
			Name:        v.Name,
			Router:      v.Router,
			Path:        v.Path,
			AppletePath: v.AppletePath,
		}
		if permission, ok := codes[v.Code]; ok &&
			permission.Router == v.Router && permission.Path == v.Path {
			ret[i].Selected = true
			usedCodes[v.Code] = codes[v.Code]
		}
		if len(v.Childs) > 0 {
			ret[i].Childs = makeNormalGroupPermission(v.Childs, codes, usedCodes)
		}
	}
	return ret
}

func checkGroupInfoParam(req *pb_v1.GroupInfoRequest) error {
	if req.Id < 1 {
		return errors.ParamsError
	}
	return nil
}

//
func GroupInfo(ctx context.Context, req *pb_v1.GroupInfoRequest) (reply *pb_v1.GroupInfoReply, err error) {

	reply = &pb_v1.GroupInfoReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()

	err = checkGroupInfoParam(req)
	if err != nil {
		return nil, err
	}

	p := &dbmodel.TGroup{}
	where := map[string]interface{}{
		"id": req.Id,
	}
	err = p.Find(database.DB(), where)
	if err != nil && err != gorm.ErrRecordNotFound {
		return nil, errors.DataBaseError
	}
	if p.ID == 0 {
		return nil, errors.ErrRecordNotFound
	}
	gardenPermissionTree, err := GardenPermissionTree(p.GardenId)
	if err != nil {
		return nil, err
	}

	if p.IsSuper == 1 {
		reply.List = makeSuperGroupPermission(gardenPermissionTree)
		return reply, nil
	}

	codes := map[string]Permission{}
	usedCodes := map[string]Permission{}
	json.Unmarshal([]byte(p.PermissionCodes), &codes)

	permissions := makeNormalGroupPermission(gardenPermissionTree, codes, usedCodes)
	reply.List = permissions

	// 云营后台删改权限造成与已绑定的权限不一致，需删除变动的权限
	if len(usedCodes) != len(codes) {
		group := dbmodel.TGroup{}
		where := map[string]interface{}{
			"id": req.Id,
		}
		bytes, _ := json.Marshal(usedCodes)
		values := map[string]interface{}{
			"permission_codes": string(bytes),
		}
		group.Update(database.DB(), where, values)
	}

	return reply, nil
}
