// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package permission

import (
	"context"
	"encoding/json"
	"fmt"
	"git.getensh.com/common/gopkgs/cache"
	"git.getensh.com/common/gopkgs/database"
	"git.getensh.com/common/gopkgs/logger"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
	"gorm.io/gorm"
	"property-system/errors"
	dbmodel "property-system/model"
	pb_v1 "property-system/pb/v1"
	"strings"
	"time"
)

const SystemPermissionChangeKeyPrefix = "system_permission_change_"

// 权限改变后，用户端会查看是否变化，触发重新登录
func PermissionChanged(users []dbmodel.TUser) error {
	for _, v := range users {
		key := fmt.Sprintf("%s%d", SystemPermissionChangeKeyPrefix, v.ID)
		_, err := cache.Redis().Set(key, fmt.Sprintf("%d", time.Now().Unix()))
		if err != nil {
			return errors.RedisError
		}
	}
	return nil
}

func UserPermissionTime(uid int64) string {
	key := fmt.Sprintf("%s%d", SystemPermissionChangeKeyPrefix, uid)
	str, _ := cache.Redis().Get(key)
	return str
}

const GlobalPermissionKey = "property_global_permission"

func GloablePermissionTime(gardenId int64) string {
	key := fmt.Sprintf("%s%d", GlobalPermissionKey, gardenId)
	str, _ := cache.Redis().Get(key)
	return str
}

func GlobalPermissionTimeCache(gardenId int64) error {
	key := fmt.Sprintf("%s%d", GlobalPermissionKey, gardenId)
	_, err := cache.Redis().Set(key, fmt.Sprintf("%d", time.Now().Unix()))
	if err != nil {
		return errors.RedisError
	}
	return nil
}

func checkGroupUpdateParam(req *pb_v1.GroupUpdateRequest) error {
	switch {
	case len(req.PermissionCodes) == 0:
		return status.Error(10003, "权限不能为空")
	case req.GardenId < 1:
		return status.Error(10003, "小区不能为空")
	case req.Cid < 1:
		return status.Error(10003, "物业公司不能为空")
	case req.GroupName == "":
		return status.Error(10003, "权限组名称不能为空")
	case req.Id < 1:
		return status.Error(10003, "权限组不能为空")

	}
	return nil
}

//
func GroupUpdate(ctx context.Context, req *pb_v1.GroupUpdateRequest) (reply *pb_v1.GroupUpdateReply, err error) {

	reply = &pb_v1.GroupUpdateReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()

	err = checkGroupUpdateParam(req)
	if err != nil {
		return nil, err
	}
	p := dbmodel.TGroup{}
	where := map[string]interface{}{
		"id":        req.Id,
		"cid":       req.Cid,
		"garden_id": req.GardenId,
	}
	err = p.Find(database.DB(), where)
	if err != nil && err != gorm.ErrRecordNotFound {
		return nil, errors.DataBaseError
	}
	if p.ID == 0 {
		return nil, errors.ErrRecordNotFound
	}
	if p.IsSuper == 1 {
		return nil, errors.SuperGroupError
	}
	reply.Origin = &pb_v1.GroupUpdateRequest{
		Id:              req.Id,
		Cid:             req.Cid,
		GardenId:        req.GardenId,
		GroupName:       p.GroupName,
		GroupDesc:       p.GroupDesc,
		PermissionCodes: strings.Split(p.PermissionCodes, ","),
	}

	now := time.Now()

	gardenCodesM, err := GardenPermissionCodes(req.GardenId)
	if err != nil {
		return nil, err
	}
	array := []string{}
	for _, v := range req.PermissionCodes {
		if gardenCodesM[v] {
			array = append(array, v)
		}
	}
	req.PermissionCodes = array

	codes := strings.Join(req.PermissionCodes, ",")
	if len(req.PermissionCodes) == 0 {
		codes = ""
	} else {
		m, err := getPermissions(req.PermissionCodes)
		if err != nil {
			return nil, err
		}
		bytes, _ := json.Marshal(m)
		codes = string(bytes)
	}

	group := &dbmodel.TGroup{}
	values := map[string]interface{}{
		"permission_codes": string(codes),
		"is_super":         p.IsSuper,
		"group_name":       req.GroupName,
		"updated_at":       now,
		"group_desc":       req.GroupDesc,
	}
	db := database.DB().Begin()
	err = group.Update(db, where, values)

	if err != nil {
		db.Rollback()
		return nil, errors.DataBaseError
	}

	user := &dbmodel.TUser{}
	where = map[string]interface{}{
		"group_id":  req.Id,
		"cid":       req.Cid,
		"garden_id": req.GardenId,
	}
	list, err := user.List(db, where, nil, -1, -1)
	if err != nil {
		db.Rollback()
		return nil, errors.DataBaseError
	}
	err = PermissionChanged(list)
	if err != nil {
		db.Rollback()
		return nil, err
	}

	db.Commit()
	return reply, nil
}
