// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package spackage

import (
	"context"
	"encoding/json"
	"fmt"
	"git.getensh.com/common/gopkgs/database"
	"git.getensh.com/common/gopkgs/logger"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
	"property-system/errors"
	dbmodel "property-system/model"
	pb_v1 "property-system/pb/v1"
)

func checkCompanyPackageListParam(req *pb_v1.CompanyPackageListRequest) error {
	if req.PageSize == 0 {
		req.PageSize = 10
	}
	if req.Page == 0 {
		req.Page = 1
	}
	if req.Cid == 0 && !req.Management {
		return status.Error(10003, "公司id不能为空")
	}
	return nil
}

//
func CompanyPackageList(ctx context.Context, req *pb_v1.CompanyPackageListRequest) (reply *pb_v1.CompanyPackageListReply, err error) {

	reply = &pb_v1.CompanyPackageListReply{}
	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()

	err = checkCompanyPackageListParam(req)
	if err != nil {
		return nil, err
	}

	p := dbmodel.TPackageOrder{}
	where := map[string]interface{}{}

	if req.Cid > 0 {
		where["cid"] = req.Cid
	}
	if req.ForGarden {
		if req.Cid == 0 {
			return nil, errors.ParamsError
		}
		where["status"] = 2
		where["order_type"] = 1
		where["garden_id"] = 0
	} else {
		if req.GardenId > 0 {
			where["garden_id"] = req.GardenId
		}
		if req.Status > 0 {
			where["status"] = req.Status
		}
		if req.OrderType > 0 {
			where["order_type"] = req.OrderType
		}
	}

	reply.Total, err = p.Count(database.DB(), where, nil)
	if err != nil {
		return nil, errors.DataBaseError
	}
	reply.Page = req.Page
	if reply.Total == 0 {
		return reply, nil
	}

	list, err := p.List(database.DB(), where, nil, int(req.Page), int(req.PageSize))
	if err != nil {
		return nil, errors.DataBaseError
	}
	gardenIds := []int64{}
	garden := dbmodel.TGardenApproved{}
	for _, v := range list {
		if v.GardenId > 0 {
			gardenIds = append(gardenIds, v.GardenId)
		}

	}
	where = map[string]interface{}{
		"id in": gardenIds,
	}
	gardenM := map[int64]dbmodel.TGardenApproved{}
	if len(gardenIds) > 0 {
		glist, err := garden.List(database.DB(), where, nil, -1, -1)
		if err != nil {
			return nil, errors.DataBaseError
		}
		for _, v := range glist {
			gardenM[v.ID] = v
		}
	}

	reply.List = make([]*pb_v1.CompanyPackageItem, len(list))

	for i, v := range list {
		pac := pb_v1.PackageInfoReply{}
		json.Unmarshal([]byte(v.PackageContent), &pac)
		item := &pb_v1.CompanyPackageItem{
			Desc:        pac.Desc,
			Name:        pac.Name,
			Amount:      v.Amount,
			Enable:      pac.Enable,
			PackageId:   v.PackageId,
			OrderId:     v.ID,
			Status:      v.Status,
			Period:      v.Period,
			Price:       pac.Price,
			GardenId:    v.GardenId,
			Cid:         v.Cid,
			CompanyName: v.CompanyName,
			PayType:     v.PayType,
			Feedback:    v.Feedback,
			GardenName:  gardenM[v.GardenId].GardenName,
			CreatedAt:   v.CreatedAt.Unix(),
		}
		if v.OrderType == 1 {
			item.ExpireAt = gardenM[v.GardenId].ExpireAt
		}
		for _, application := range pac.Applications {
			appItem := &pb_v1.CompanyPackageApplicationItem{
				Icon:    application.Icon,
				Desc:    application.Desc,
				Name:    application.Name,
				Content: application.Content,
				Enable:  application.Enable,
			}
			item.ApplicationList = append(item.ApplicationList, appItem)
		}
		reply.List[i] = item
	}

	return reply, nil
}
