// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package spackage

import (
	"context"
	"encoding/json"
	"fmt"
	"gorm.io/gorm"
	"property-system/errors"
	dbmodel "property-system/model"
	"property-system/parser"
	"property-system/pb"
	pb_v1 "property-system/pb/v1"
	"property-system/utils"
	"time"

	"git.getensh.com/common/gopkgs/database"
	"git.getensh.com/common/gopkgs/logger"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
)

func checkPackageOrderAddParam(req *pb_v1.PackageOrderAddRequest) error {
	switch {
	case req.OrderType != 1 && req.OrderType != 2:
		return status.Error(10003, "订单类型错误")
	case req.OrderType == 1 && (req.Cid < 1 || req.CompanyName == ""):
		return status.Error(10003, "公司不能为空")
	case req.PackageId < 1:
		return status.Error(10003, "套餐不能为空")
	case req.Period < 1:
		return status.Error(10003, "周期不能为空")
	case req.OrderType == 2 && req.GardenId == 0:
		return status.Error(10003, "续费，小区不能为空")

	}
	return nil
}

func renewOrder(req *pb_v1.PackageOrderAddRequest) (reply *pb_v1.PackageOrderAddReply, err error) {
	reply = &pb_v1.PackageOrderAddReply{}

	p := dbmodel.TPackageOrder{}
	where := map[string]interface{}{
		"package_id": req.PackageId,
		"garden_id":  req.GardenId,
		"status":     2,
		"order_type": 1,
	}
	err = p.Find(database.DB(), where)
	if err != nil && err != gorm.ErrRecordNotFound {
		return nil, errors.DataBaseError
	}
	if p.ID == 0 {
		return nil, errors.ErrRecordNotFound
	}
	pac := pb_v1.PackageInfoReply{}
	json.Unmarshal([]byte(p.PackageContent), &pac)
	if pac.Price*req.Period != req.Amount {
		return nil, status.Error(10003, "金额不一致")
	}

	p.ID = 0
	p.Status = 1
	p.OrderType = 2
	p.Period = req.Period
	p.Amount = req.Amount
	p.CreatedAt = time.Now()
	p.UpdatedAt = time.Now()
	db := database.DB().Begin()
	err = p.Insert(db)

	if err != nil {
		db.Rollback()
		return nil, errors.DataBaseError
	}

	if req.ShouldAccess {
		mreq := pb_v1.PackageOrderApproveRequest{OrderId: p.ID, Status: true, Feedback: "直接通过"}
		err = ApproveHandle(&mreq, db)
		if err != nil {
			db.Rollback()
			return nil, err
		}
	}
	db.Commit()
	if !req.ShouldAccess {
		utils.RobotMsg(fmt.Sprintf("有新的套餐订单，请关注"))
	}
	return reply, nil
}

//
func PackageOrderAdd(ctx context.Context, req *pb_v1.PackageOrderAddRequest) (reply *pb_v1.PackageOrderAddReply, err error) {

	reply = &pb_v1.PackageOrderAddReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()

	err = checkPackageOrderAddParam(req)
	if err != nil {
		return nil, err
	}

	if req.OrderType == 2 {
		return renewOrder(req)
	}

	now := time.Now()

	p := dbmodel.TPackageOrder{}

	mreq := pb_v1.PackageInfoRequest{Id: req.PackageId}
	mreply, err := pb.Common.PackageInfo(ctx, &mreq)
	if err != nil {
		return nil, err
	}
	if mreply.Price*req.Period != req.Amount {
		return nil, status.Error(10003, "金额不一致")
	}

	p.CreatedAt = now
	p.UpdatedAt = now
	p.PackageId = req.PackageId
	p.Amount = req.Amount
	p.Cid = req.Cid
	p.CompanyName = req.CompanyName
	p.Status = 1
	p.PayType = 1
	p.OrderType = 1
	p.Period = req.Period
	bytes, _ := json.Marshal(*mreply)
	p.PackageContent = string(bytes)
	db := database.DB().Begin()
	err = p.Insert(db)
	if err != nil {
		db.Rollback()
		return nil, errors.DataBaseError
	}

	if req.ShouldAccess {
		mreq := pb_v1.PackageOrderApproveRequest{OrderId: p.ID, Status: true, Feedback: "直接通过"}
		err = ApproveHandle(&mreq, db)
		if err != nil {
			db.Rollback()
			return nil, err
		}
	}

	db.Commit()
	reply.OrderId = p.ID
	reply.Period = req.Period
	reply.Amount = p.Amount
	reply.GardenCount = 1
	reply.PayType = 1
	reply.BankName = parser.Conf.Bank.BankName
	reply.BankAccount = parser.Conf.Bank.BankAccount
	reply.BankUserName = parser.Conf.Bank.BankUserName
	if !req.ShouldAccess {
		utils.RobotMsg(fmt.Sprintf("有新的套餐订单，请关注"))
	}
	return reply, nil
}
