// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package user

import (
	"context"
	"encoding/json"
	"fmt"
	"git.getensh.com/common/gopkgs/util"
	"property-system/consts"
	"property-system/errors"
	dbmodel "property-system/model"
	"property-system/pb"
	pb_v1 "property-system/pb/v1"
	"strings"
	"time"

	"git.getensh.com/common/gopkgs/database"
	"git.getensh.com/common/gopkgs/logger"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
)

func checkUserAddParam(req *pb_v1.UserAddRequest) error {
	switch {
	case req.Password == "":
		return status.Error(10003, "密码不能为空")
	case req.Phone == "":
		return status.Error(10003, "联系人电话不能为空")
	case req.UserName == "":
		return status.Error(10003, "账户名不能为空")
	case req.RealName == "":
		return status.Error(10003, "真实姓名不能为空")
	case req.GardenId < 1 || req.GardenName == "":
		return status.Error(10003, "小区不能为空")
	case req.Cid < 1:
		return status.Error(10003, "物业公司不能为空")
	case !req.ByCompany && req.Uid < 1:
		return status.Error(10003, "当前uid不能为空")
	case req.BasePermission == 0:
		return status.Error(10003, "业务权限不能为空")
	case req.GroupId == 0:
		return status.Error(10003, "管理后台权限组不能为空")

	}
	return nil
}

//
func UserAdd(ctx context.Context, req *pb_v1.UserAddRequest) (reply *pb_v1.UserAddReply, err error) {

	reply = &pb_v1.UserAddReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()

	err = checkUserAddParam(req)
	if err != nil {
		return nil, err
	}

	if req.Uid > 0 {
		/*
			p := &dbmodel.TUser{}
			where := map[string]interface{}{
				"id":req.Uid,
			}
			err = p.Find(database.DB(), where)
			if err != nil && err != gorm.ErrRecordNotFound {
				return nil, errors.DataBaseError
			}
			if p.ID == 0 {
				return nil, errors.ErrRecordNotFound
			}
			req.GardenId = p.GardenId
			req.GardenName = p.GardenName
		*/
	}

	now := time.Now()

	passwd, _ := util.GetMd5Pass(req.Password, consts.CRYPTO_KEY)
	user := dbmodel.TUser{
		GardenId:       req.GardenId,
		Cid:            req.Cid,
		User:           req.UserName,
		Phone:          req.Phone,
		Email:          req.Email,
		Password:       passwd,
		GardenName:     req.GardenName,
		UpdatedAt:      now,
		CreatedAt:      now,
		GroupId:        req.GroupId,
		RealName:       req.RealName,
		DepartmentId:   req.DepartmentId,
		BasePermission: req.BasePermission,
		Gender:         req.Gender,
	}

	db := database.DB().Begin()
	err = user.Insert(db)
	if err != nil {
		db.Rollback()
		if strings.Contains(strings.ToLower(err.Error()), "duplicate") {
			return nil, errors.UserExist
		}
		return nil, errors.DataBaseError
	}
	openimId := fmt.Sprintf("property%d", user.ID)
	mreq := pb_v1.OpenImRegisterRequest{UserId: openimId, Platform: 5, NickName: req.RealName}
	_, err = pb.Thirdparty.OpenImRegister(ctx, &mreq)
	if err == nil {
		where := map[string]interface{}{
			"id": user.ID,
		}
		values := map[string]interface{}{
			"openim_id": openimId,
		}
		err = user.Update(db, where, values)
		if err != nil {
			db.Rollback()
			return nil, errors.DataBaseError
		}
	}
	db.Commit()
	reply.Id = user.ID

	return reply, nil
}
