// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package user

import (
	"context"
	"encoding/json"
	"fmt"
	"git.getensh.com/common/gopkgs/database"
	"git.getensh.com/common/gopkgs/logger"
	"git.getensh.com/common/gopkgs/util"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
	"property-system/consts"
	"property-system/errors"
	"property-system/impl/v1/permission"
	dbmodel "property-system/model"
	pb_v1 "property-system/pb/v1"
)

func getGardenInfos(ids []int64) (gardens map[int64]dbmodel.TGardenApproved, err error) {
	gardens = map[int64]dbmodel.TGardenApproved{}
	if len(ids) == 0 {
		return gardens, nil
	}
	p := &dbmodel.TGardenApproved{}
	where := map[string]interface{}{
		"id in": ids,
	}
	list, err := p.List(database.DB(), where, nil, -1, -1)
	if err != nil {
		return nil, errors.DataBaseError
	}

	for _, v := range list {
		gardens[v.ID] = v
	}

	return gardens, nil
}

//
func LoginByPhone(ctx context.Context, req *pb_v1.LoginByPhoneRequest) (reply *pb_v1.LoginByPhoneReply, err error) {

	reply = &pb_v1.LoginByPhoneReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()

	if req.Phone == "" {
		return reply, errors.ParamsError
	}

	p := &dbmodel.TUserGroup{}
	db := database.DB()

	where := map[string]interface{}{
		"phone":       req.Phone,
		"garden_id >": 0,
	}
	if req.Password != "" {
		passwd, _ := util.GetMd5Pass(req.Password, consts.CRYPTO_KEY)
		where["password"] = passwd
	}

	//  查找用户是否存在
	list, err := p.List(db, where, nil, -1, -1)
	if err != nil {
		return reply, errors.DataBaseError
	}
	if len(list) == 0 {
		return nil, errors.ErrRecordNotFound
	}

	if req.Applete {
		newList := []dbmodel.TUserGroup{}
		for _, v := range list {
			if v.BasePermission == BasePermissionBack {
				continue
			}
			newList = append(newList, v)
		}
		if len(newList) == 0 {
			return nil, status.Error(10003, "没有小程序权限")
		}
		list = newList
	} else {
		newList := []dbmodel.TUserGroup{}
		for _, v := range list {
			if v.BasePermission == BasePermissionApplete {
				continue
			}
			newList = append(newList, v)
		}
		if len(newList) == 0 {
			return nil, status.Error(10003, "没有管理后台权限")
		}
		list = newList
	}

	gardenIds := []int64{}
	for _, v := range list {
		gardenIds = append(gardenIds, v.GardenId)
	}
	gardenM, err := getGardenInfos(gardenIds)
	if err != nil {
		return nil, err
	}

	reply.List = make([]*pb_v1.LoginReply, len(list))
	for i, p := range list {
		openimId := ""
		if p.OpenimId == "" {
			openimId = openimRegister(p.ID, p.RealName, p.Gender)
		}
		if gardenM[p.GardenId].Enable == 2 {
			continue
		}
		gardenPermissionTree := permission.GardenPermissionTreeFromRedis(p.GardenId)
		if len(gardenPermissionTree) == 0 {
			gardenPermissionTree, err = permission.GardenPermissionTreeFromDb(p.GardenId)
			if err != nil {
				return nil, err
			}
		}
		item := &pb_v1.LoginReply{}
		if p.IsSuper == 1 {
			item.Permissions = makeSuperGroupPermission(gardenPermissionTree)
		} else {
			codes := map[string]permission.Permission{}
			usedCodes := map[string]permission.Permission{}
			json.Unmarshal([]byte(p.PermissionCodes), &codes)
			permissions := makeNormalGroupPermission(gardenPermissionTree, codes, usedCodes)

			// 云营后台删改权限造成与已绑定的权限不一致，需删除变动的权限
			/*if len(usedCodes) != len(codes) {

				group := dbmodel.TGroup{}
				where := map[string]interface{}{
					"id": p.GroupId,
				}
				bytes, _ := json.Marshal(usedCodes)
				values := map[string]interface{}{
					"permission_codes": string(bytes),
				}
				group.Update(database.DB(), where, values)
				codes = nil
				usedCodes = nil
			}*/
			item.Permissions = permissions
		}

		item.Uid = p.ID
		item.GardenId = p.GardenId
		item.Cid = p.Cid
		item.GroupId = p.GroupId
		item.OpenimId = openimId

		if p.IsSuper == 1 {
			item.IsSuperGroup = true
		}
		item.UserName = p.User
		item.GardenName = p.GardenName
		item.GlobalPermissionTime = permission.GloablePermissionTime(p.GardenId)
		item.UserPermissionTime = permission.UserPermissionTime(p.ID)
		reply.List[i] = item
	}

	if len(list) > 0 && len(reply.List) == 0 {
		return nil, status.Error(10003, "小区已停用")
	}

	return reply, nil
}
