// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package user

import (
	"context"
	"encoding/json"
	"fmt"
	"git.getensh.com/common/gopkgs/database"
	"git.getensh.com/common/gopkgs/logger"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
	"gorm.io/gorm"
	"property-system/errors"
	"property-system/impl/v1/permission"
	dbmodel "property-system/model"
	pb_v1 "property-system/pb/v1"
)

func PermissionRegetByCompany(ctx context.Context, req *pb_v1.PermissionRegetRequest) (reply *pb_v1.PermissionRegetReply, err error) {

	reply = &pb_v1.PermissionRegetReply{}
	if req.GardenId < 0 {
		return nil, errors.ParamsError
	}

	gardenInfo, err := getGardenInfo(req.GardenId)
	if err != nil {
		return nil, err
	}
	if !gardenInfo.InUse {
		return nil, status.Error(10003, "小区已删除")
	}

	// 获取权限信息
	gardenPermissionTree := permission.GardenPermissionTreeFromRedis(gardenInfo.ID)
	if len(gardenPermissionTree) == 0 {
		gardenPermissionTree, err = permission.GardenPermissionTreeFromDb(gardenInfo.ID)
		if err != nil {
			return nil, err
		}
	}
	if err != nil {
		return nil, err
	}
	reply.GardenName = gardenInfo.GardenName
	reply.Permissions = makeSuperGroupPermission(gardenPermissionTree)

	reply.GardenEnable = true
	reply.IsSuperGroup = true
	reply.GlobalPermissionTime = permission.GloablePermissionTime(req.GardenId)

	return reply, nil
}

//
func PermissionReget(ctx context.Context, req *pb_v1.PermissionRegetRequest) (reply *pb_v1.PermissionRegetReply, err error) {

	reply = &pb_v1.PermissionRegetReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()
	if req.Company {
		return PermissionRegetByCompany(ctx, req)
	}
	if req.Uid == 0 {
		return reply, errors.ParamsError
	}

	p := &dbmodel.TUserGroup{}
	db := database.DB()

	where := map[string]interface{}{
		"id": req.Uid,
	}

	//  查找用户是否存在
	err = p.Find(db, where)
	if err != nil {
		if err == gorm.ErrRecordNotFound {
			return reply, errors.UserNotExist
		}
		return reply, errors.DataBaseError
	}
	if p.ID == 0 {
		return nil, status.Error(10003, "该用户不存在")
	}
	if p.GardenId == 0 {
		return nil, status.Error(10003, "未绑定小区")
	}
	if req.Applete && p.BasePermission == BasePermissionBack {
		return nil, status.Error(10003, "没有小程序权限")
	}
	if !req.Applete && p.BasePermission == BasePermissionApplete {
		return nil, status.Error(10003, "没有管理后台权限")
	}

	// 获取小区信息
	gardenInfo, err := getGardenInfo(p.GardenId)
	if err != nil {
		return nil, err
	}
	if !gardenInfo.InUse {
		return nil, status.Error(10003, "小区已删除")
	}
	if gardenInfo.Enable == 2 {
		return nil, status.Error(10003, "小区已停用")
	}
	// 获取权限信息
	gardenPermissionTree := permission.GardenPermissionTreeFromRedis(gardenInfo.ID)
	if len(gardenPermissionTree) == 0 {
		gardenPermissionTree, err = permission.GardenPermissionTreeFromDb(gardenInfo.ID)
		if err != nil {
			return nil, err
		}
	}

	if err != nil {
		return nil, err
	}
	if p.IsSuper == 1 {
		reply.Permissions = makeSuperGroupPermission(gardenPermissionTree)
	} else {
		codes := map[string]permission.Permission{}
		usedCodes := map[string]permission.Permission{}
		json.Unmarshal([]byte(p.PermissionCodes), &codes)
		permissions := makeNormalGroupPermission(gardenPermissionTree, codes, usedCodes)

		// 运营后台删改权限造成与已绑定的权限不一致，需删除变动的权限
		/*
			if len(usedCodes) != len(codes) {

				group := dbmodel.TGroup{}
				where := map[string]interface{}{
					"id":p.GroupId,
				}
				bytes, _ := json.Marshal(usedCodes)
				values := map[string]interface{}{
					"permission_codes":string(bytes),
				}
				group.Update(database.DB(), where, values)
				codes = nil
				usedCodes = nil
			}*/
		reply.Permissions = permissions
	}

	reply.Uid = p.ID
	reply.GardenId = p.GardenId
	reply.Cid = p.Cid
	reply.GroupId = p.GroupId
	reply.GardenEnable = true
	reply.UserPermissionTime = permission.UserPermissionTime(p.ID)
	reply.GlobalPermissionTime = permission.GloablePermissionTime(p.GardenId)
	if p.IsSuper == 1 {
		reply.IsSuperGroup = true
	}
	reply.GardenName = p.GardenName

	return reply, nil
}
