// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package user

import (
	"context"
	"encoding/json"
	"fmt"
	"git.getensh.com/common/gopkgs/database"
	"git.getensh.com/common/gopkgs/logger"
	"git.getensh.com/common/gopkgs/util"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
	"gorm.io/gorm"
	"property-system/consts"
	"property-system/errors"
	"property-system/impl/v1/permission"
	dbmodel "property-system/model"
	pb_v1 "property-system/pb/v1"
	"strings"
	"time"
)

func checkUserUpdateParam(req *pb_v1.UserUpdateRequest) error {
	switch {
	case req.Phone == "":
		return status.Error(10003, "联系人电话不能为空")
	case req.UserName == "":
		return status.Error(10003, "账户名不能为空")
	case req.GardenId < 1 || req.GardenName == "":
		return status.Error(10003, "小区不能为空")
	case req.RealName == "":
		return status.Error(10003, "真实姓名不能为空")
	case req.Cid < 1:
		return status.Error(10003, "物业公司不能为空")
	case req.Id < 1:
		return status.Error(10003, "用户id不能为空")
	case !req.ByCompany && req.Uid < 1:
		return status.Error(10003, "当前uid不能为空")
	case req.BasePermission == 0:
		return status.Error(10003, "业务权限不能为空")
	case req.GroupId == 0:
		return status.Error(10003, "管理后台权限组不能为空")

	}
	return nil
}

func checkPersonUpdateParam(req *pb_v1.UserUpdateRequest) error {
	switch {
	case req.Phone == "":
		//return status.Error(10003, "联系人电话不能为空")
	case req.UserName == "":
		//return status.Error(10003, "账户名不能为空")
	case req.RealName == "":
		//return status.Error(10003, "真实姓名不能为空")
	case req.Id < 1:
		return status.Error(10003, "用户id不能为空")

	}
	return nil
}

func PersonUpdate(ctx context.Context, req *pb_v1.UserUpdateRequest) (reply *pb_v1.UserUpdateReply, err error) {
	reply = &pb_v1.UserUpdateReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()

	err = checkPersonUpdateParam(req)
	if err != nil {
		return nil, err
	}

	now := time.Now()

	where := map[string]interface{}{
		"id": req.Id,
	}
	values := map[string]interface{}{

		//"email":      req.Email,
		//"user":       req.UserName,
		//"phone":      req.Phone,
		"updated_at": now,
		//"real_name":  req.RealName,
	}
	if req.UserName != "" {
		//values["user"] = req.UserName
	}
	if req.RealName != "" {
		//values["real_name"] = req.RealName
	}
	if req.Phone != "" {
		values["phone"] = req.Phone
	}
	if req.Email != "" {
		values["email"] = req.Email
	}
	if req.Gender > 0 {
		values["gender"] = req.Gender
	}

	src := &dbmodel.TUser{}
	err = src.Find(database.DB(), where)
	if err != nil && err != gorm.ErrRecordNotFound {
		return nil, errors.DataBaseError
	}
	if src.ID == 0 {
		return nil, errors.ErrRecordNotFound
	}

	if req.Uid == src.ID && !req.ByCompany {
		if req.GroupId != src.GroupId || req.BasePermission != src.BasePermission {
			return nil, errors.UserSelfError
		}
	}

	reply.Origin = &pb_v1.UserUpdateRequest{
		Id:           req.Id,
		Cid:          src.Cid,
		GardenId:     src.GardenId,
		GroupId:      src.GroupId,
		UserName:     src.User,
		Email:        src.Email,
		Phone:        src.Phone,
		RealName:     src.RealName,
		DepartmentId: src.DepartmentId,
	}

	user := dbmodel.TUser{}

	err = user.Update(database.DB(), where, values)
	if err != nil {
		if strings.Contains(strings.ToLower(err.Error()), "duplicate") {

			return nil, errors.UserExist
		}

		return nil, errors.DataBaseError
	}

	return reply, nil
}

//
func UserUpdate(ctx context.Context, req *pb_v1.UserUpdateRequest) (reply *pb_v1.UserUpdateReply, err error) {

	if req.Person {
		return PersonUpdate(ctx, req)
	}
	reply = &pb_v1.UserUpdateReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()

	err = checkUserUpdateParam(req)
	if err != nil {
		return nil, err
	}

	now := time.Now()

	passwd := ""
	if req.Password != "" {
		passwd, _ = util.GetMd5Pass(req.Password, consts.CRYPTO_KEY)
	}

	where := map[string]interface{}{
		"id":  req.Id,
		"cid": req.Cid,
	}
	values := map[string]interface{}{
		"garden_id":     req.GardenId,
		"email":         req.Email,
		"user":          req.UserName,
		"phone":         req.Phone,
		"updated_at":    now,
		"garden_name":   req.GardenName,
		"real_name":     req.RealName,
		"department_id": req.DepartmentId,
		"gender":        req.Gender,
	}
	if req.GroupId > 0 {
		values["group_id"] = req.GroupId
	}

	if req.BasePermission > 0 {
		values["base_permission"] = req.BasePermission
	}
	if passwd != "" {
		values["password"] = passwd
	}

	src := &dbmodel.TUser{}
	err = src.Find(database.DB(), where)
	if err != nil && err != gorm.ErrRecordNotFound {
		return nil, errors.DataBaseError
	}
	if src.ID == 0 {
		return nil, errors.ErrRecordNotFound
	}

	if req.Uid == src.ID && !req.ByCompany {
		if req.GroupId != src.GroupId || req.BasePermission != src.BasePermission {
			return nil, errors.UserSelfError
		}
	}

	reply.Origin = &pb_v1.UserUpdateRequest{
		Id:           req.Id,
		Cid:          req.Cid,
		GardenId:     src.GardenId,
		GroupId:      src.GroupId,
		UserName:     src.User,
		Email:        src.Email,
		Phone:        src.Phone,
		RealName:     src.RealName,
		DepartmentId: src.DepartmentId,
	}

	user := dbmodel.TUser{}
	db := database.DB().Begin()
	err = user.Update(db, where, values)
	if err != nil {
		if strings.Contains(strings.ToLower(err.Error()), "duplicate") {
			db.Rollback()
			return nil, errors.UserExist
		}
		db.Rollback()
		return nil, errors.DataBaseError
	}

	if src.GroupId != req.GroupId {
		err = permission.PermissionChanged([]dbmodel.TUser{{ID: src.ID}})
		if err != nil {
			db.Rollback()
			return nil, err
		}
	}

	db.Commit()

	return reply, nil
}
